/*    
 *    Copyright 2008 Anssi Piirainen
 *
 *    This file is part of FlowPlayer.
 *
 *    FlowPlayer is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Foobar is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with FlowPlayer.  If not, see <http://www.gnu.org/licenses/>.
 */
import org.flowlib.EventDispatcher;
import org.flowplayer.ClipDescriptor;
import org.flowplayer.config.Config;
import org.flowplayer.FlowPlayer;
import org.flowplayer.skin.Skin;
import org.flowplayer.Util;

import LuminicBox.Log.ConsolePublisher;
import LuminicBox.Log.Logger;

/**
 * Abstract baseclass for movie clips.
 */
class org.flowlib.NewAbstractMovieClip extends MovieClip {
	public static var logger:Logger;
	private var loader:MovieClipLoader;
	private var childClips:Array;
	var __width:Number;
	var __height:Number;
	var __visible:Boolean = true;
	private var dispatcher:EventDispatcher;
	private var txtFormat:TextFormat;
	private var loadingSymbols:Boolean;
	private var firstArrangeDone:Boolean = false;

	public function NewAbstractMovieClip() {
		this._visible = false;
		loadingSymbols = false;
		logger = new Logger();
		logger.addPublisher(new ConsolePublisher());
		logger.debug("NewAbstractMovieClip::constructor this._name: " + this._name);
		
		loader = new MovieClipLoader();
		loader.addListener(this);
		
		dispatcher = new EventDispatcher();
		initClip();
		createChildren();
		loadClips(getChildDescriptors());
		__visible = true;
	}
	
	function removeChildClip(clip:MovieClip):Void {
		childClips = Util.removeFromArray(childClips, clip);
		clip.removeMovieClip();
	}
	
	private function initClip():Void {
//		__width = _width;
//		__height = _height;
		_xscale = 100;
		_yscale = 100;
		_x = 0;
		_y = 0;
	}
	
	private function loadClips(clipDescriptors:Array):Void {
//		logger.debug(this._name + "::loadClips:");
//		logger.debug(clipDescriptors);
		childClips = new Array();
		for (var i : Number = 0; i < clipDescriptors.length; i++) {
			var descriptor:ClipDescriptor = clipDescriptors[i];
			childClips.push(loadOrCreateClip(descriptor));
		}
				
		// Arrange right away if only using embedded symbols for this clip
		if (! loadingSymbols) {
//			logger.info("All children are loaded for " + this._name);
			allChildrenCreated(childClips);
			arrange();
			if (__visible)
				this._visible = true;
		} else { 
			logger.info("Waiting for child clips to load for parent " + this._name);
		}
	}
	
	public function onLoadInit(target:MovieClip):Void {
//		logger.debug("onLoadInit(), target " + target);
		loadInit(target);
		if (target == this) return;
		if (firstArrangeDone) return; 
		if (allChildrenLoaded() && areChildrenCreated()) {
//			logger.debug("onLoadInit(); all children available --> arranging " + this);
			allChildrenCreated(childClips);
			doArrange();
			if (__visible)
				this._visible = true;
			loadingSymbols = false;
			firstArrangeDone = true;
		}
	}

	private function loadInit(target:MovieClip) : Void {
		// possibly overridden in subclasses		
	}
	
	private function allChildrenLoaded():Boolean {
		return clipsLoaded(childClips);
	}
	
	private function areChildrenCreated():Boolean {
		// possibly overridden in subclasses
		return true;
	}
	
	private function allChildrenCreated(clips:Array):Void {
		// possibly overridden in subclasses
	}

	public function loadOrCreateClip(descriptor:ClipDescriptor):MovieClip {
//		logger.debug(this._name + "::loadOrCreateClip:");
//		logger.debug(descriptor);
		if (descriptor.isEmptyMovie()) {
			return createEmptyMovieClip(descriptor.getName(), getNextHighestDepth());
		} else if (loadableClip(descriptor)) {
			loadingSymbols = true;
//			logger.debug("loading image from file " + descriptor.getURL());
			return loadImageAsClip(descriptor.getName(), descriptor.getURL());
		} else {
			var clip:MovieClip = attachMovie(descriptor.getSymbolName(), descriptor.getName(), getNextHighestDepth()); 
//			logger.info("===============> Attached " + clip + "  from symbol " + descriptor.getSymbolName());
			clip.loaded = true;
			return clip;
		}
	}
	
	private function loadableClip(descriptor:ClipDescriptor):Boolean {
		if (descriptor == undefined) return false;
		if (descriptor.getURL() != null && ! useEmbeddedImages()) return true;
		if (descriptor.getSymbolName() == null) return true;
		return false;
	}
	
	private function createChildren():Void {
		// possibly overridden in subclasses
	}
	
	private function getChildDescriptors():Array {
//		throw new Error("getChildDescriptors(): This method must be overridden in a subclass");
		return new Array();		
	}

	public function set width(newWidth:Number):Void {
		setSize(newWidth, null);
	}
	
	public function get width():Number {
		return __width;
	}

	public function set height(newHeight:Number):Void {
		setSize(null, newHeight);
	}
	
	public function get height():Number {
		return __height;
	}
	
	public function set visible(isVisible:Boolean):Void {
		__visible = isVisible;
		_visible = isVisible;
	}
	
	public function setSize(nW:Number, nH:Number, forceArrange:Boolean):Void {
		logger.debug("setSize on " + this._name);
		if (! forceArrange) {
			if (nW == __width && nH == __height) {
				logger.debug(this._name + " setSize(), size not changing --> returning without arranging");
				return;
			}
			if (! (nW > 0 || nH > 0)) { 
				logger.debug(this._name + " setSize(), new size (width, height) == (" + nW + ", " + nH + ") --> returning without arranging");
				return;
			}
		}

		_xscale = 100;
		_yscale = 100;
		if (nW != null && nW != undefined) {
			__width = nW;
			logger.debug(this._name + " __widht set to " + __width);
		}
		if (nH != null && nH != undefined) {
			__height = nH;
			logger.debug(this._name + " __height set to " + __height);
		}
		
		onSetSize();
		doArrange();
	}
	
	private function onSetSize():Void {
		// possibly overridden in subclasses
	}
	
	private function doArrange():Void {
//		if (__width == 0 || __height == 0) return;
//		logger.info("calling arrange on " + this);
		arrange();
	}
	
	function arrange():Void {
		throw new Error("arrange(): This method must be overridden in a subclass");
	}
			
	function arrangeClip(clip:Object, x:Number, y:Number, width:Number, height:Number):Void {
		Util.arrangeClip(clip, x, y, width, height);
	}

	private function arrangeTextFieldsVertically(header:TextField, field:TextField, another:TextField, margin:Number, gap:Number):Void {
		header._x = margin;
		header._y = another._y + another._height + gap;
		if (field)
			field._x = header._x + header._width + gap;
		field._y = header._y;
	}

	function addEventListener(typeName:String, listener:Object):Void {
		dispatcher.addEventListener(typeName, listener);
	}
		
	function dispatchEvent(typeName:String, eventObject:Object):Void {
		dispatcher.dispatchEvent(typeName, eventObject);
	}
	
	static function drawRectangleWithBorder(clip:MovieClip, x:Number, y:Number, width:Number, height:Number, lineThickness:Number, lineRGB:Number):Void {
		Util.drawRectangleWithBorder(clip, x, y, width, height, lineThickness, lineRGB);
	}
		
	static function drawRectangle(clip:MovieClip, x:Number, y:Number, width:Number, height:Number, drawBorder:Boolean):Void {
		Util.drawRectangle(clip, x, y, width, height, drawBorder);
	}
		
	function isMouseOverClip(clip:MovieClip):Boolean {
		return clip.hitTest(_root._xmouse, _root._ymouse, true);
	}
		
	function clipsLoaded(clips:Array):Boolean {
		if (clips == null) return true;
		logger.debug("checking clips in " + clips +  " for size");
		for (var i = 0; i < clips.length; i++) {
			var clip:MovieClip = clips[i];
			if (! clip.loaded) {
//				logger.error("clip " + clips[i] + " size is " + clips[i]._width);
				if (clip == null || clip == undefined) { 
//					logger.info("haveSize(): child clip of " + this._name + " in position " + i + " is null or undefined");
					return false;
				}
				if (! (clip._width > 0)) {
//					logger.info("Clip " + clip + " does not have size, " + getDimensions(clip));
					return false;
				}
			}
		}
//		logger.fatal("all clips in " + clips +  " have size");
		return true;
	}
	
	function maxHeight(clips:Array):Number {
		return Util.maxHeight(clips);
	}
	
	function isLoaded(clip:MovieClip):Boolean {
		return clipsLoaded([clip]);
	}
	
	function getDefaultTextFormat():TextFormat {
		if (txtFormat == null) {
			txtFormat = new TextFormat();
			txtFormat.size = 12;
			//txtFormat.font = "vera";
		}
		return txtFormat;
	}
		
	function traceDims(clip:MovieClip, name:String) {
		logger.debug("DIMENSIONS FOR clip " + clip._name);
		logger.debug("    - width " + clip._width);
		logger.debug("    - height " + clip._height);
		logger.debug("    - X pos " + clip._x);
		logger.debug("    - Y pos " + clip._y);
	}
	
	function getDimensions(clip:MovieClip) : String {
		return Util.getDimensions(clip);
	}
				
	private function loadImageAsClip(clipName:String, fileURL:String):MovieClip {
		NewAbstractMovieClip.logger.info("Loading image " + fileURL + ", clipName = " + clipName);
		var clip:MovieClip = createEmptyMovieClip(clipName, getNextHighestDepth());
		
		// store some properties so that we can later identify this clip
		clip.name = clipName;
		clip.url = fileURL;
		
		loader.loadClip(fileURL, clip);
		return clip;
	}
	
	
	public function getConfig():Config {
		return FlowPlayer.getConfig();
	}
	
	function getSkin():Skin {
		if (! getConfig().getSkin()) return Skin.create(getConfig(), "airlight");
		return getConfig().getSkin();
	}
	
	function useEmbeddedImages():Boolean {
		return getConfig().useEmbeddedButtonImages();
	}

	private function createText(id:String, text:String, wordWrap:Boolean, multiline:Boolean,selectable:Boolean, width:Number, height:Number, fontSize:Number, fonts:String):TextField {
		return Util.createText(this, getSkin(), id, text, wordWrap, multiline, selectable, width, height, fontSize, fonts);
	}

}