/*    
 *    Copyright 2008 Anssi Piirainen
 *
 *    This file is part of FlowPlayer.
 *
 *    FlowPlayer is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Foobar is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with FlowPlayer.  If not, see <http://www.gnu.org/licenses/>.
 */
import org.flowlib.Observable;
import org.flowplayer.FlowPlayerExternalInterface;
import org.flowplayer.MediaContext;
import org.flowplayer.MediaControl;
import org.flowplayer.PlaybackStatus;
import org.flowplayer.playlist.Clip;
import org.flowplayer.thumbs.Thumbnail;
import org.flowplayer.Util;

import LuminicBox.Log.ConsolePublisher;
import LuminicBox.Log.Logger;
import flash.external.ExternalInterface;
import org.flowplayer.playlist.PlayList;
import org.flowplayer.config.Config;

/**
 * Abstract Controller for FLV playback.
 */
class org.flowplayer.FLVController extends Observable implements MediaControl {
	private var logger:Logger;
	private var protectionCodeLoader:LoadVars;
	private var paused:Boolean = false;
	private var playing:Boolean = false;
	private var metaDataReceived:Boolean = false;
	
	var videoContainer:Video;
	var videoMc:MovieClip;
	var nc:NetConnection = null;
	var ns:NetStream;
	var sound:Sound;
	var videoDurationSecs:Number = 0;
	var buffering:Boolean = false;
	private var thumbLocation:String;
	private var duractionTrackingIntervalId:Number;
	private var done:Boolean;

	private var clip:Clip;
	private var lastStreamTime:Number;
	private var origVolume:Number;
	private var keyFrames:Object;
	private var cuePoints:Array;
	private var previousCuePoint:Object;
	private var config:Config;

	/**
	 * Creates a new controller.
	 * 
	 * @param videoContainerCompoent
	 * @param playList
	 * @param bufferLengthSeconds
	 */	
	public function FLVController(config:Config) {
		super();
		logger = new Logger();
		this.config = config;
		protectionCodeLoader = new LoadVars();
		this.protectionCodeLoaded.controller = this;
		protectionCodeLoader.onData = this.protectionCodeLoaded;

		logger.addPublisher ( new ConsolePublisher() );
		
		resetState();
		this.duractionTrackingIntervalId = -1;
	}	
	
	public function attachView(videoMc:MovieClip, mediaType:String):Void {
		this.videoMc = videoMc;
//		connect();
	}
	
	private function connect():Void {
		nc = new NetConnection();
		logger.info("connecting to '" + config.getStreamingServerURL() + "'");
		nc.connect(config.getStreamingServerURL());
		
		ns = new NetStream(nc);
		logger.info("FLVController:: setting initial buffer length to " + getStartingBufferLength() + " seconds");
		ns.setBufferTime(getStartingBufferLength());
		videoContainer.attachVideo(ns);

		MovieClip(videoMc).attachAudio(ns);
//		sound = new Sound(videoMc);

		this.cuePointCallback.context = this;
		Object(ns).onCuePoint = this.cuePointCallback;

		ns.onStatus = this.netStreamStatusCallback;
		Object(ns).onPlayStatus = this.onPlayStatus;
		this.netStreamStatusCallback.streamer = this;
		this.onPlayStatus.streamer = this;
		
		ns.onMetaData = this.netStreamMetaDataCallback;
		this.netStreamMetaDataCallback.streamer = this;		
	}
	
	private function mute():Void {
		if (! origVolume)
			origVolume = sound.getVolume();
		sound.setVolume(0);
	}
	
	private function resetState():Void {
		this.playing = false;
		this.paused = false;
		this.buffering = false;
	}
	
	public function reset():Void {
		stop();
		connect();
	}

	public function attachVideo(video:Video, mediaType:String) : Void {
		this.videoContainer = video;
	}

	/**
	 * @see MediaControl.startBuffering()
	 */
	public function startBuffering(clip:Clip):Void {
		if (buffering) return;
		buffering = true;
		this.clip = clip;
		startLoading();
	}

	/**
	 * @see MediaControl.playCurrent()
	 */
	public function playCurrent() : Void {
		throw new Error("Unsupported method!");
	}

	/**
	 * @see MediaControl.play()
	 */
	public function play(clip:Clip):Void {
		logger.debug("FLVController::play");
		logStateVariables();
		playing = true;

		if (this.clip == clip && buffering && paused) {
			resume();
			this.paused = false;
			return;
		}

		this.clip = clip;
		paused = false;
		buffering = false;
		videoContainer.clear();
		startLoading();
	}
	
	private function logStateVariables():Void {
		logger.debug("FLVController.playing: " + playing);
		logger.debug("FLVController.paused: " + paused);
		logger.debug("FLVController.buffering: " + buffering);
	}

	private function startLoading():Void {
		this.metaDataReceived = false;
		this.done = false;

//		var clip:Clip = playList.currentClip();
		this.videoDurationSecs = clip.getTotalDuration();
		logger.debug("Clip's total duration is " + this.videoDurationSecs + " parent duration is " + clip.getDuration());

		var clips:Array = clip.getChildren();
		clips.unshift(clip);
		dispatchEventWithObj("onStartBuffering", this.clip);
		loadClips(clips);		
	}
	
	private function loadClips(clips:Array):Void {
		logger.debug("Starting a stream with following clips:");
		logger.debug(clips);
		mute();
		createDurationTrackingInterval();

		if (anyClipProtected(clips)) {
			this.clipsToLoadTemp = clips;
			loadProtectedClips();
		} else
			doLoadClips(clips);
	}
	
	public var clipsToLoadTemp:Array;

	private function loadProtectedClips():Void {
		protectionCodeLoader.load(config.getProtectionFileName());
	}
	
	private function protectionCodeLoaded(code):Void {
		if (code)
			arguments.callee.controller.logger.info("received protection code " + code);
		else {
			arguments.callee.controller.logger.fatal("unable to load protection code");
			return;
		}
		var controller:FLVController = arguments.callee.controller;
		controller.doLoadClips(controller.clipsToLoadTemp, code);
	}
	
	public function doLoadClips(clips:Array, protectionCode:String):Void {
		connect();
		for (var i : Number = 0; i < clips.length; i++) {
			var clip:Clip = clips[i];
			
			if (clip.isLive()) {
				logger.debug("starting a live stream");
				ns.play(clip.getURL(protectionCode));
			} else if (config.getStreamingServerURL() != undefined) {
				startStreamingClip(clip, protectionCode, i == 0);
			} else {
				logger.debug("loading video from " + clip.getURL(protectionCode));
				ns.play(clip.getURL(protectionCode));
			}			
		}		
	}
	
	private function startStreamingClip(clip:Clip, protectionCode:String, resetPlayList:Boolean):Void {
		var start:Number = clip.getStartTime();
		var duration:Number = clip.getDuration() || 0;
		var url:String = config.getStreamingServerURL() ? clip.getFilename(protectionCode) : clip.getURL(protectionCode); 
		logger.debug("streaming from '" + url + "'");
		
		if (start >= 0 && duration > 0) {
			ns.play(url, start, duration, resetPlayList);
		} else {
			ns.play(url, 0, -1, resetPlayList);
		}
	}
	
	private function anyClipProtected(clips:Array):Boolean {
		for (var i : Number = 0; i < clips.length; i++) {
			var clip:Clip = clips[i];
			if (clip.isProtected())
				return true;
		}
		return false;
	}
	
	private function createDurationTrackingInterval():Void {
		checkProgress.controller = this;
		if (this.duractionTrackingIntervalId != -1) return;
		this.duractionTrackingIntervalId = setInterval(checkProgress, 200);
	}
	
	private function deleteDurationTrackingInterval():Void {
		clearInterval(this.duractionTrackingIntervalId);
		this.duractionTrackingIntervalId = -1;
	}

	private function checkProgress():Void {
		arguments.callee.controller.doCheckProgress();
	}
	
	private function doCheckProgress():Void {
		if (! playing || paused) return;
		if (duractionTrackingIntervalId == -1) return;
		
		checkAndFireCuePoints(getStreamTime());

		if (playedWholeDuration()) {
			logger.debug("FLVController.doCheckProgress:: done == " + done + ", duration == " + videoDurationSecs);
			//stop();
			deleteDurationTrackingInterval();
			dispatchEvent("onClipDone");
			// the done flag is set by the NetStream callback, we need to reset it to false here for the next stream
			done = false;
		}
		lastStreamTime = getStreamTime();
	}
	
	private function playedWholeDuration():Boolean {
		var curStreamTime:Number = getStreamTime();		

		if ((videoDurationSecs > 0 && curStreamTime >= videoDurationSecs) || done)
			return true;
		if (videoDurationSecs > 0 && curStreamTime == lastStreamTime && (videoDurationSecs - curStreamTime) < .1 )
			return true;
		return false;
	}
	
	/**
	 * @see MediaControl.stop()
	 */
	public function stop():Void {
		logger.debug("FLVController::stop(): stopping!");
		resetState();
		ns.close();
		deleteDurationTrackingInterval();
	}
	
	public function rewind():Void {
		logger.debug("FLVController::rewind()");
		if (! playing) {
			stop();
			return;
		}
		ns.pause(true);
		playing = false;
		paused = true;
		buffering = true;
		seek(0);
	}
	
	/**
	 * Pauses playback.
	 */
	public function pause():Void {
		if (! playing) return;
		logger.debug("FLVController::pause(): pausing");
		this.paused = true;
		ns.pause(true);
	}

	private function setIsPaused(newIsPaused:Boolean):Void {
		paused = newIsPaused;
	}
	
	/**
	 * @see MediaControl.isPaused()
	 */
	public function isPaused():Boolean {
		return paused;
	}
	
	/**
	 * @see MediaControl.resume()
	 */
	public function resume():Void {
		logger.debug("FLVController::resume()");
		if (! this.paused) {
			play();
		} else {
			this.paused = false;
		}
		dispatchEventWithObj("onLoadBegin", clip);		
		this.playing = true;
		restoreVolume();
		ns.pause(false);
		buffering = false;
	}
	
	/**
	 * Toggles pause/resume.
	 */
//	public function pauseOrResume():Void {
//		logger.debug("FLVController::pauseOrResume, paused " + this.paused);
//		this.paused = ! this.paused;
//		ns.pause();
//	}
	
	/**
	 * Handles callbacks from NetStream.
	 */
	function netStreamStatusCallback(info:Object):Void {
		arguments.callee.streamer.updateNetStreamStatus(info);
	}

	function onPlayStatus(info:Object):Void {
		FLVController(arguments.callee.streamer).updatePlayStatus(info);
	}

	function updatePlayStatus(info:Object):Void {
		logger.info("FLVController:: NetStream.status, nc.url " + nc.uri);
		logger.info(traceProps(info));
		
		if (info.code.toString() == "NetStream.Play.Complete") {
			done = true;
		}		
	}

	private function traceProps(info:Object):Void {
		for (var prop:Object in info) {
            logger.info("\t" + prop + ":\t" + info[prop]);
        }
	}

	function updateNetStreamStatus(info:Object):Void {
		logger.info("FLVController:: NetStream.status, nc.url '" + nc.uri + "'");
		logger.info(traceProps(info));
		// make sure we are tracking progress
		createDurationTrackingInterval();

		if (info.code.toString() == "NetStream.Play.Stop") {
			if (! isRTMPStreaming())
				done = true;

		} else if (info.code.toString() == "NetStream.Seek.Notify") {
//			if (config.getStreamingServerURL())
			dispatchEventWithObj("onStartBuffering", this.clip);

		} else if (info.code.toString() == "NetStream.Play.Start") {
			setBufferLength(getStartingBufferLength());
			dispatchEventWithObj("onLoadBegin", clip);		

		} else if (info.code.toString() == "NetStream.Buffer.Empty") {
			this.ns.setBufferTime(getStartingBufferLength());
			if (! playedWholeDuration())
				dispatchEventWithObj("onStartBuffering", this.clip);

		} else if (info.code.toString() == "NetStream.Buffer.Full") {
			setBufferLength(config.getMainBufferLength());
			dispatchEventWithObj("onBufferFull", this.clip);
//			restoreVolume();

		} else if (info.code.toString() == "NetStream.Buffer.Flush") {
			setBufferLength(getStartingBufferLength());
			if (! playedWholeDuration())
				dispatchEventWithObj("onBufferFlush", this.clip);

		} else if (info.code.toString() == "NetStream.Play.StreamNotFound") {
			logger.fatal("Cannot find Stream, NetConnection.uri is '" + nc.uri + "''");
			dispatchEventWithObj("onStreamNotFound", clip);		
		}
	}
	
	private function restoreVolume():Void {
		if (sound.getVolume() != 0) return;
		sound.setVolume(origVolume == 0 ? 0 : origVolume || 100);
	}

	private function setBufferLength(length:Number):Void {
		logger.info("Swithcing to buffer lentgh " + length + ", ns.bufferTime now = " + ns.bufferTime);
		if (ns.bufferTime != length) {
			logger.debug("Swithcing to buffer lentgh " + length);
			ns.setBufferTime(length);
		}
	}
	
	/**
	 * Handles metadata callbacks from NetStream. This is used
	 * to detect the FLV duration.
	 */
	function netStreamMetaDataCallback(obj:Object):Void {
		var player:FLVController = arguments.callee.streamer;
		player.updateNetStreamMetaData(obj);
	}
	
	function updateNetStreamMetaData(obj:Object):Void {
		if (obj.duration == undefined) return;
		if (! isPlaying()) {
			logger.debug("FLVController::metadata received, pausing player");
			ns.pause(true);
			//ns.seek(0);
			this.paused = true;
		}
		
		this.keyFrames = obj.keyframes;
		
//		if (this.metaDataReceived) {
//			logger.debug("updateNetStreamMetaData(): MetaData already recived for this clip, returning"); 
//			dispatchEvent("onLoadBegin");		
//			return;
//		}
//		this.metaDataReceived = true;
		
		logger.info("this.videoDurationSecs == " + this.videoDurationSecs);
		if (! this.videoDurationSecs)
			this.videoDurationSecs = obj.duration;
		
		if (obj.duration)
			clip.setDurationFromMetaData(obj.duration);

		logger.debug("FLVController::FLV duration: " + obj.duration + " sec.");
		logger.debug("FLVController::FLV videodatarate: " + obj.videodatarate + " Kbit/s");
		logger.debug("FLVController::FLV audiodatarate: " + obj.audiodatarate + " Kbit/s");
		logger.debug("FLVController::FLV creationdate: " + obj.creationdate);

		var cuePoints:Object = obj.cuePoints;
		if (cuePoints != undefined) {
			logger.info("Cue points received!" );
			this.thumbLocation = obj.thumbLocation;
			obj.cuePoints = parseCuePoints(thumbLocation, cuePoints);
			dispatchEventWithObj("onMetaData", obj);
		} else {
			logger.info("No cue points received");
			dispatchEventWithObj("onMetaData", obj);
		}
		
		onNetStreamMetaData(obj);
		logger.error("dispatching onLoadBegin()");
		dispatchEventWithObj("onLoadBegin", clip);		
		
		if (isPlaying()) {
			restoreVolume();
		}
	}
	
	private function onNetStreamMetaData(arg0 : Object) : Void {
		// possibly overridden in subclasses
	}

	/**
	 * Seeks into the specified position. This position should be
	 * within the range of the length currently loaded in the buffer.
	 * 
	 * @param percentage a value between 0 and the value returned by {@link getBufferPercentage()}
	 * @see MediaControl.getProgressPercentage()
	 */
	public function seek(percentage:Number):Void {
		if (percentage < 0) {
			percentage = 0;
		}
		if (percentage > 100) {
			percentage = 100;
		}
		var targetSeconds:Number = Math.floor((percentage/100) * videoDurationSecs);
		logger.debug("FLVController::seeking to " + targetSeconds + " seconds (" + percentage + " %)");
		
		if (config.getStreamingServer() == 'lighttpd')
			seekToKeyFrameClosestTo(percentage);
		else
			seekToTime(targetSeconds);
		
		lastStreamTime = targetSeconds;
	}

	public function seekToTime(offset : Number) : Void {
		logger.debug("FLVController.seekTotime");
		if (videoDurationSecs == 0) {
			return;
		}
		if (config.getStreamingServer() == 'lighttpd') { 
			var targetFrame:Number = findFrame(0, keyFrames.times.length,offset); 
			logger.debug("FLVController::seekToTime(lighttpd) " + offset + " (frame:"+ targetFrame+ ", position "+keyFrames.filepositions[targetFrame]+")"); 
			ns.play(clip.getURL() + "?start=" + keyFrames.filepositions[targetFrame]); 
		} else { 
			var targetOffset:Number = offset; 
			if (targetOffset < 0) 
				targetOffset = 0; 
			if (targetOffset > videoDurationSecs) 
				targetOffset = videoDurationSecs; 
			ns.seek(targetOffset);
		}
	}

	private function seekToKeyFrameClosestTo(percentage:Number):Void { 
		var seekPosition:Number = percentage * videoDurationSecs / 100; 
		var i:Number = findFrame(0, keyFrames.times.length,seekPosition); 
		logger.debug("FLVController::seekToKeyFrame " + seekPosition+" "+i+" "+keyFrames.filepositions[i]); 
		ns.play(clip.getURL() + "?start=" + keyFrames.filepositions[i]); 
	}
	
	/** 
	 * Find the frame number which is closest to the target offset time in seconds.  
	 */ 
	private function findFrame(rangeBegin:Number, rangeEnd:Number, target:Number):Number { 
		if (rangeBegin == rangeEnd) return rangeBegin; 
		
		var rangeMid = Math.floor((rangeEnd + rangeBegin)/2); 
		if (keyFrames.times[rangeMid] >= target)
			return findFrame(rangeBegin,rangeMid,target); 
		else
			return findFrame(rangeMid+1,rangeEnd,target); 
	}

	/**
	 * Returns 'flv'.
	 */
	public function getFormat():String {
		return 'flv';
	}
	
	/**
	 * @see MediaControl.isPlaying()
	 */
	public function isPlaying():Boolean {
		return this.playing;
	}
	
	private function setIsPlaying(newIsPlaying:Boolean):Void {
		this.playing = newIsPlaying;
	}
	
	/**
	 * @see MediaControl#onClipChanged()
	 */
	public function onClipChanged():Void {
	}
	
	function parseCuePoints(thumbLocation:String, cuePoints:Object):Array {
		var thumbs:Array = new Array();
		logger.info("cue points length = " + cuePoints.length);
		for (var i : Number = 0; i < cuePoints.length; i++) {
			var oCue:Object = cuePoints[i];
			logger.debug("cuepoint: " + oCue.name + ", type " + oCue.type + ", thumb + " + oCue.parameters.thumb);
			if (oCue.type == "navigation" && oCue.parameters.thumb != undefined)
				thumbs.push(createThumbnail(thumbLocation, oCue.parameters.thumb, oCue.time));
		}
		thumbs.sort(Thumbnail.compare);
		for (var i : Number = 0; i < thumbs.length; i++) {
			thumbs[i].pos = i;
		}
		return thumbs;
	}

	private function createThumbnail(baseURL:String, fileName:String, time:Number):Thumbnail {
		return new Thumbnail(Util.addBaseURL(baseURL, fileName), time);
	}

	private function cuePointCallback(cuePointInfo:Object):Void {
		// for some strange reason passing the cuePointInfo straight will not deliver it correctly, constructing a new object instead
		dispatchOnCuePoint(cuePointInfo);
		
		arguments.callee.context.logger.info("cuePointCallback, type " + cuePointInfo.type + ", thumb " + cuePointInfo.parameters.thumb + ", paused " + arguments.callee.context.paused);
		if (arguments.callee.context.paused) return;
		arguments.callee.context.logger.debug("time: " + cuePointInfo.time);
		if (cuePointInfo.type != "navigation") return;
		var thumb:Object = cuePointInfo.parameters.thumb;
		if (thumb == undefined)	return;
		arguments.callee.context.logger.info("dispatching onCuePointWithThumb");
		arguments.callee.context.dispatchEventWithObj("onCuePointWithThumb", cuePointInfo.time);
	}

	public function getPlaybackStatus() : PlaybackStatus {
		var duration:Number = videoDurationSecs || clip.getDuration() || 0;
		
		if (config.getStreamingServerURL()) {
//			logger.debug("getPlaybackStatus: " + ns.bufferTime + " / " + ns.bufferLength);
			return new PlaybackStatus(duration, getStreamTime(), ns.bufferTime, ns.bufferLength, true);
		} else {
//			logger.debug("getPlaybackStatus: " + ns.bytesLoaded + " / " + ns.bytesTotal);
			return new PlaybackStatus(duration, getStreamTime(), ns.bytesTotal, ns.bytesLoaded, false);
		}
	}
	
	private function getStreamTime(clip:Clip):Number {
		var time:Number = ns.time - (config.getStreamingServer() == 'red5' ? clip.getStartTime() : 0);
		if (time <= 0)
			time = 0;
		return time;
	}

	public function setContext(context:MediaContext) {
	}
	
	/**
	 * Toggles looping.
	 */ 
	public function toggleLoop():Void {
	}

	/**
	 * Is this plauer looping?
	 */ 
	public function isLooping():Boolean {
		return false;
	}

	public function stopCurrent() : Void {
		stop();
	}

	private function isRTMPStreaming():Boolean {
		return config.getStreamingServerURL().toUpperCase().indexOf("RTMP", 0) == 0;
	}

	public function addCuePoint(cueTimeSeconds:Number, cueName:String, cueParameters:Object):Void {
		logger.debug("adding cuePoint to time " + cueTimeSeconds);
		if (! this.cuePoints)
			this.cuePoints = new Array();
		var cue:Object = new Object();
		cue.time = cueTimeSeconds;
		cue.name = cueName;
		cue.parameters = cueParameters;
		cue.type = "actionscript";
		cuePoints[cueTimeSeconds] = cue;
		//{ time: cuePointInfo.time, type: cuePointInfo.type, thumb: cuePointInfo.parameters.thumb, parameters: cuePointInfo.parameters }
	}

	private function checkAndFireCuePoints(streamTime:Number):Void {
		var timeRounded:Number = Math.round(streamTime);
		if (streamTime >= timeRounded && cuePoints[timeRounded]) {
			var cue:Object = cuePoints[timeRounded];
			if (alreadyFired(cue, streamTime))
				return;
			previousCuePoint = cue;
			logger.info("cuePointReached:");
			logger.info(cue);
			dispatchOnCuePoint(cue);
		}				
	}
	
	private function dispatchOnCuePoint(cuePointInfo:Object):Void {
		FlowPlayerExternalInterface.call("onCuePoint", { name: cuePointInfo.name, time: cuePointInfo.time, type: cuePointInfo.type, thumb: cuePointInfo.parameters.thumb, parameters: cuePointInfo.parameters });
	}

	public function addCuePoints(cuePoints:Array):Void {
		for (var i:Number = 0; i < cuePoints.length; i++) {
			var cue:Object = cuePoints[i];
			addCuePoint(cue.time, cue.name, cue.parameters);
		}
	}

	private function alreadyFired(currentCuePoint:Object, streamTime:Number):Boolean {
		return currentCuePoint == previousCuePoint && streamTime - currentCuePoint.time <= 1;
	}

	public function setPlayList(playList:PlayList):Void {
	}

	private function getStartingBufferLength():Number {
		var configValue:Number = config.getStartingBufferLength();
		if (configValue) return configValue;
		return config.getMainBufferLength();
	}

	public function setSound(sound : Sound) : Void {
		this.sound = sound;
	}
	
	public function onVolumeChanged(volumePercentage:Number):Void {
		this.origVolume = volumePercentage;
	}

	public function setViewVisibility():Boolean {
		return false;
	}

	public function allowCenterViewOnLoadBegin() : Boolean {
		return false;
	}

}
