﻿/*    
 *    Copyright 2008 Anssi Piirainen
 *
 *    This file is part of FlowPlayer.
 *
 *    FlowPlayer is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Foobar is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with FlowPlayer.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.flowplayer.config.*;
import org.flowplayer.playlist.PlayList;
import org.flowplayer.playlist.Clip;
import org.flowplayer.skin.Skin;
import org.flowplayer.thumbs.Thumbnail;
import org.flowplayer.Util;
import org.flowplayer.MediaSize;

import LuminicBox.Log.ConsolePublisher;
import LuminicBox.Log.Logger;
import cinqetdemi.JSON;

/**
 * AppContext is used to lookup things like configuration settings.
 */
class org.flowplayer.config.HierarchicalConfig implements ExtendedConfig {
  	private var config:Object;
  	private var childConfig:ExtendedConfig;
  	private var playList:PlayList;
  	private var logger:Logger;
	private var name:String;
	private var thumbs:Array;
  	
  	public function HierarchicalConfig(configObj:Object, name:String) {
  		logger = new Logger();
  		logger.addPublisher(new ConsolePublisher());
  		this.name = name || "HierarchicalConfig with " + configObj;
		initConfig(configObj);
  		logger.info("Configuration object for '" + name + "' (" + config + ")");
  		logger.info(config);
  		if (config["childConfig"]) {
  			logger.info("adding child config");
  			addChild(new HierarchicalConfig(config["childConfig"]));
  		}
  	}
  	
	private function initConfig(configObj:Object):Void {
		logger.debug("initConfig: " + configObj);
		if (typeof(configObj) == "string") {
			try {
				logger.debug("parsing " + configObj);
				config = JSON.parse(String(configObj));
			} catch (e:Error) {
				logger.error("error while parsing config '" + name + "': " + e);
			}
		} else
			config = configObj;
	}
	
	public function stringify():String {
		var configTemp:ExtendedConfig = childConfig;
		childConfig = undefined;
		var result:String = JSON.stringify(config);
		childConfig = configTemp;
		return result;
	}
	
	/**
	* Adds a new configuration as a child config of this config object. If a
	* certain setting is not defined in this config, it will be searched from
	* the child config. Again if the setting is not found from the provided
	* child config, it will be searched from it's child and so on. 
	* 
	* The search will end into {@see DefaultsConfig} where all settings are defined
	* with default values. This happens if the setting was not found from the config
	* chain.
	* 
	* @param	defaults the defaults to be added
	*/
  	public function addChild(childConfig:ExtendedConfig):Void {
		logger.debug("addChild, current childConfig is " + this.childConfig);
  		if (this.childConfig)
  			this.childConfig.addChild(childConfig);
  		else
  			this.childConfig = childConfig;
  	}
	
	public function getChild():Config {
		return childConfig || DefaultsConfig.getInstance();
	}
  	
 	public function getConfigFileName():String {
 		return this.config.configFileName;
 	}
  	
  	public function setConfigFileName(fileName:String):Void {
  		this.config.configFileName = fileName;
  	}

	public function getBaseURL():String {
		if (getStreamingServerURL() != undefined) return "";
		if (this.config.baseURL == undefined) return getChild().getBaseURL();
		logger.debug(this.name + " has baseURL " + this.config.baseURL + ", will use that");
		return this.config.baseURL;
	}
		
	public function setBaseURL(baseURL:String):Void {
		this.config.baseURL = baseURL;
	}
	
	public function isAutoBuffering():Boolean {
		if (this.config.autoBuffering == undefined) return getChild().isAutoBuffering();
		return this.config.autoBuffering;
	}
	
	public function isAutoPlay():Boolean {
		if (this.config.autoPlay == undefined) return getChild().isAutoPlay();
		return this.config.autoPlay;
	}
 
	public function isAutoRewind() : Boolean {
		if (this.config.autoRewind == undefined) return getChild().isAutoRewind();
		return this.config.autoRewind;
	}
	
 	public function isLooping():Boolean {
 		if (this.config.loop == undefined) return getChild().isLooping();
 		return true == this.config.loop;
 	}
	
	public function enableAds():Boolean {
 		if (this.config.enableAds == undefined) return getChild().enableAds();
 		return true == this.config.enableAds;
	}
 	
 	public function getStartingBufferLength():Number {
		if (this.config.startingBufferLength) return parseInt(this.config.startingBufferLength);
		var defaultVal:Number = getChild().getStartingBufferLength();
		return defaultVal || getMainBufferLength();
 	}
 	
 	public function getMainBufferLength():Number {
 		if (this.config.bufferLength == undefined) return getChild().getMainBufferLength();
 		return parseInt(this.config.bufferLength);
 	}

 	public function getVideoHeight() : Number {
 		if (isFullScreen()) return undefined;
 		if (this.config.videoHeight == undefined) return getChild().getVideoHeight();
		return parseInt(this.config.videoHeight);
	}
 	public function getControlsWidth():Number {
 		if (this.config.controlsWidth == undefined) return getChild().getControlsWidth();
		var width:Number = parseInt(this.config.controlsWidth);
		return width > Stage.width ? Stage.width : width;
 	}

	public function setControlsWidth(newWidth:Number):Void {
		this.config.controlsWidth = newWidth;
	}
 	
 	public function showControls():Boolean {
 		if (this.config.hideControls == undefined) return getChild().showControls();
		return ! this.config.hideControls;
 	}
	
	public function showStopButton():Boolean {
 		if (this.config.showStopButton == undefined) return getChild().showStopButton();
		return this.config.showStopButton;
 	}
	
	public function showPlayButton():Boolean {
 		if (this.config.showPlayButton == undefined) return getChild().showPlayButton();
		return this.config.showPlayButton;
 	}
	
	public function showFullScreenButton():Boolean {
 		if (this.config.showFullScreenButton == undefined) return getChild().showFullScreenButton();
		return this.config.showFullScreenButton;
 	}
	
	public function showMenu():Boolean {
 		if (this.config.showMenu == undefined) return getChild().showMenu();
		return this.config.showMenu;
 	}
 	
 	public function showPlayListButtons():Boolean {
 		if (! this.config.showPlayListButtons) {
 			return getChild().showPlayListButtons();
 		}
		return this.config.showPlayListButtons;
 	}
 
	public function showVolumeSlider() : Boolean {
 		if (this.config.showVolumeSlider == undefined) return getChild().showVolumeSlider();
		return this.config.showVolumeSlider;
	}

	public function showMuteVolumeButton() : Boolean {
 		if (this.config.showMuteVolumeButton == undefined) return getChild().showMuteVolumeButton();
		return this.config.showMuteVolumeButton;
	}
  	
  	public function showScrubber():Boolean {
 		if (this.config.showScrubber == undefined) return getChild().showScrubber();
		return this.config.showScrubber;
  	}
  	
	public function useEmbeddedButtonImages():Boolean {
		if (this.config.useEmbeddedButtonImages == undefined) return getChild().useEmbeddedButtonImages();
		return this.config.useEmbeddedButtonImages;
	}
 	
 	public function getSkin(rootConfig:Config):Skin {
		return getChild().getSkin(rootConfig || this);
 	}
 	
 	public function getPlayList(baseURL:String):PlayList {
 		if (this.playList != undefined) {
			logger.debug("returning cached playlist:");
			logger.debug(this.playList);
			return this.playList;
		}
 		
 		// if videoFile is defined, it will be used to create a playList for one clip
  		if (getVideoFile() != undefined) {
			this.playList = PlayList.forOneClip(null, baseURL || getBaseURL(), getVideoFile(), usePlayOverlay(), getSplashImageFile());
  		} else if (config.playList != undefined) {
			logger.debug("creating playlist, using big play overlay: " + usePlayOverlay());
			this.playList = PlayList.fromObjectsWithProps(baseURL || getBaseURL(), config.playList, usePlayOverlay(), getSplashImageFile(), getStreamingServerURL() != undefined);
 		} else {
 			this.playList = getChild().getPlayList(baseURL || getBaseURL());
  		}
		return this.playList;
 	}

	public function getVideoFile(useBaseUrl:Boolean):String {
		if (getStreamingServerURL() != undefined) {
			return this.config.videoFile || getChild().getVideoFile(); 
		}
		if (useBaseUrl == undefined || useBaseUrl)
			return Util.addBaseURL(getBaseURL(), this.config.videoFile || getChild().getVideoFile(false));
		else
			return this.config.videoFile || getChild().getVideoFile(false);
	}

	public function getNoVideoClip():Clip {
		logger.info(this.name + " noVideoClip = " + this.config.noVideoClip);
		if (! this.config.noVideoClip)
			return getChild().getNoVideoClip();
			
		this.config.noVideoClip.name = "not_found_clip";
		var list:PlayList = PlayList.fromObjectsWithProps(getBaseURL(), [this.config.noVideoClip], usePlayOverlay(), undefined, getStreamingServerURL() != undefined);
		return list.currentClip();
	}

	public function getSkinImagesBaseURL() : String {
		if (this.config.skinImagesBaseURL == undefined) return getChild().getSkinImagesBaseURL();
		return this.config.skinImagesBaseURL;
	}

	public function getSplashImageFile():String {
		if (this.config.splashImageFile == undefined) return getChild().getSplashImageFile();
		return this.config.splashImageFile;
	}

	public function thumbsOnFLV() : Boolean {
		if (this.config.thumbsOnFLV == undefined) return getChild().thumbsOnFLV();
		return this.config.thumbsOnFLV;
	}

	public function getThumbnails() : Array {
		if (thumbs == undefined) {
			if (config.thumbs == undefined) return getChild().getThumbnails();
			
			logger.debug("config has thumbnails");
			var thumbsData:Array = config.thumbs;
			 
			thumbs = new Array();
			for (var i : Number = 0; i < thumbsData.length; i++) {
				var thumb:Object = thumbsData[i];
				thumbs.push(new Thumbnail(Util.addBaseURL(config.thumbLocation, thumb.thumbNail), thumb.time));
			}
			logger.debug("parsed following thumbs from config:");
			logger.debug(thumbs);
		}
		return thumbs;
	}

	public function getStreamingServerURL() : String {
		if (clipHasNetConnectionTargetURI(this.playList.currentClip())) {
			return this.playList.currentClip().getBaseURL();
		}
		if (this.config.streamingServerURL == undefined) return getChild().getStreamingServerURL();
		return this.config.streamingServerURL;
	}
	
	private function clipHasNetConnectionTargetURI(clip:Clip):Boolean {
		return clip.getBaseURL().indexOf("rtmp") == 0 || clip.getBaseURL().indexOf("RTMP") == 0; 
	}

	public function useConfigInject():Boolean {
		return this.config.configInject;
	}

	public function getStreamingServer() : String {
		if (this.config.streamingServer == undefined) return getChild().getStreamingServer();
		return this.config.streamingServer;
	}

	public function getInitialMediaSize():Number {
		logger.debug("getInitialMediaSize() " + this.config.initialScale);
		if (this.config.initialScale == undefined) return getChild().getInitialMediaSize();
		if (this.config.initialScale == 'fit') return MediaSize.FITTED_PRESERVING_ASPECT_RATIO;
		if (this.config.initialScale == 'half') return MediaSize.HALF_FROM_ORIGINAL;
		if (this.config.initialScale == 'orig') return MediaSize.ORIGINAL;
		if (this.config.initialScale == 'scale') return MediaSize.FILLED_TO_AVAILABLE_SPACE;
	}
	
	public function setIninitalMediaSize(newSize:Number):Void {
		if (newSize == MediaSize.FITTED_PRESERVING_ASPECT_RATIO) {
			this.config.initialScale = 'fit';
			return;
		}
		if (newSize == MediaSize.HALF_FROM_ORIGINAL) {
			this.config.initialScale = 'half';
			return;
		}
		if (newSize == MediaSize.ORIGINAL) {
			this.config.initialScale = 'orig';
			return;
		}
		if (newSize == MediaSize.FILLED_TO_AVAILABLE_SPACE) {
			this.config.initialScale = 'scale';
			return;
		}
	}
	
	
	public function toString():String {
		return "HierarchicalConfig, name = '" + name + "'";
	}

	public function getProtectionFileName():String {
		if (this.config.authFileName == undefined) return getChild().getProtectionFileName();
		return this.config.authFileName;
	}
	
	/*
	 *  Methods for skinning:
	 */
	 
 	public function getProgressIndicatorColor1():Number {
 		return this.config.progressBarColor1 == undefined ? getChild().getProgressIndicatorColor1() : this.config.progressBarColor1;
 	}
 	
 	public function getProgressIndicatorColor2():Number {
 		return this.config.progressBarColor2 == undefined ? getChild().getProgressIndicatorColor2() : this.config.progressBarColor2;
 	}
 	
 	public function getBufferIndicatorColor1():Number {
 		return this.config.bufferBarColor1 == undefined ? getChild().getBufferIndicatorColor1() : this.config.bufferBarColor1;
 	}
 	
 	public function getBufferIndicatorColor2():Number {
 		return this.config.bufferBarColor2 == undefined ? getChild().getBufferIndicatorColor2() : this.config.bufferBarColor2;
 	}
 	
 	public function getProgressBarBorderColor1():Number {
 		return this.config.progressBarBorderColor1== undefined ? getChild().getProgressBarBorderColor1() : this.config.progressBarBorderColor1;
 	}
 	
 	public function getProgressBarBorderColor2():Number {
 		return this.config.progressBarBorderColor2 == undefined ? getChild().getProgressBarBorderColor2() : this.config.progressBarBorderColor2;
 	}
	
	public function getBufferingAnimationColor():Number {
 		return this.config.bufferingAnimationColor == undefined ? getChild().getBufferingAnimationColor() : this.config.bufferingAnimationColor;
 	}

	public function getControlBarBackgroundColor():Number {
 		return this.config.controlBarBackgroundColor == undefined ? getChild().getControlBarBackgroundColor() : this.config.controlBarBackgroundColor;		
	}
	
	public function getTimeDisplayFontColor():Number {
 		return this.config.timeDisplayFontColor == undefined ? getChild().getTimeDisplayFontColor() : this.config.timeDisplayFontColor;
	}
	
	public function getProgressBarHeight():Number {
 		return this.config.progressBarHeight == undefined ? getChild().getProgressBarHeight() : this.config.progressBarHeight;		
	}
	
	public function getProgressBarAreaHeight():Number {
 		return this.config.progressBarAreaHeight == undefined ? getChild().getProgressBarAreaHeight() : this.config.progressBarAreaHeight;		
	}
	
	public function showMenuItem(itemNumber:Number):Boolean {
 		if (this.config.menuItems == undefined) return getChild().showMenuItem(itemNumber);
 		return this.config.menuItems[itemNumber];		
	}

	public function getFullScreenScriptURL():String {
		if (this.config.fullScreenScriptURL == undefined) return getChild().getFullScreenScriptURL();
		return this.config.fullScreenScriptURL;
	}

	public function isFullScreen():Boolean {
		if (useFlashNativeFullScreen())
			return Stage["displayState"] == "fullScreen";
		if (this.config.fullScreen == undefined) return getChild().isFullScreen();
		return this.config.fullScreen;
	}

	public function setFullScreen(fullScreen:Boolean):Void {
		if (fullScreen == undefined) 
			delete this.config.fullScreen;
		this.config.fullScreen = fullScreen;
	}

	public function getEmbedWidth():Number {
		if (this.config.embedWidth == undefined) return getChild().getEmbedWidth();
		return this.config.embedWidth;
	}

	public function getEmbedHeight():Number {
		if (this.config.embedHeight == undefined) return getChild().getEmbedHeight();
		return this.config.embedHeight;
	}

	public function setEmbedWidth(width:Number):Void {
		this.config.embedWidth = width;
	}

	public function setEmbedHeight(height:Number):Void {
		this.config.embedHeight = height;
	}

	public function setIsEmbedded(embedded:Boolean):Void {
		this.config.embedded = embedded;
	}

	public function isEmbedded():Boolean {
		return this.config.embedded;
	}

	public function getProperty(propertyName:String):Object {
		if (config[propertyName] == undefined) return getChild().getProperty(propertyName);
		return config[propertyName];
	}

	function useFlashNativeFullScreen() : Boolean {
		if (this.config.useNativeFullScreen == undefined) return getChild().useFlashNativeFullScreen();
		return this.config.useNativeFullScreen;
	}
		
	public function useHwScaling():Boolean {
		if (this.config.useHwScaling == undefined) return getChild().useHwScaling();
		return this.config.useHwScaling;
	}
	

	public function getConfigObj() : Object {
		return this.config;
	}

 	public function useSmoothing():Boolean {
 		if (this.config.useSmoothing == undefined) return getChild().useSmoothing();
 		return this.config.useSmoothing;
 	}
 	
	public function getRtmpConfigUrl() : String {
		return this.config.rtmpConfigUrl;
	}

	public function getRtmpConfigParam():Object {
		return this.config.rtmpConfigParam;
	}

	public function setPlayList(playList:PlayList):Void {
		this.playList = playList;
	}

	public function getGoogleAnalyticsPrefix() : String {
		if (this.config.googleAnalyticsPrefix == undefined) return getChild().getGoogleAnalyticsPrefix();
		return this.config.googleAnalyticsPrefix;
	}

	public function getEmbedCode() : String {
		if (this.config.embedCode == undefined) return getChild().getEmbedCode();
		return this.config.embedCode;
	}

	public function getAdServerUrl() : String {
		if (this.config.adServerUrl == undefined) return getChild().getAdServerUrl();
		return this.config.adServerUrl;
	}

	public function getAdServerAuth() : String {
		if (this.config.adServerAuth == undefined) return getChild().getAdServerAuth();
		return this.config.adServerAuth;
	}

	public function getPublisherId() : String {
		if (this.config.publisherId == undefined) return getChild().getPublisherId();
		return this.config.publisherId;
	}

	public function getCountryCode() : String {
		if (this.config.countryCode == undefined) return getChild().getCountryCode();
		return this.config.countryCode;
	}

	public function getEmailPostUrl():String {
		if (this.config.emailPostUrl == undefined) return getChild().getEmailPostUrl();
		return this.config.emailPostUrl;
	}

	public function getVideoLink() : String {
		if (this.config.videoLink == undefined) return getChild().getVideoLink();
		return this.config.videoLink;
	}

	public function getEmailVideoLink() : String {
		if (this.config.emailVideoLink == undefined) return getChild().getEmailVideoLink();
		return this.config.emailVideoLink;
	}

	public function getEmailDefaultMessage():String {
		if (this.config.emailDefaultMessage == undefined) return getChild().getEmailDefaultMessage();
		return this.config.emailDefaultMessage;
	}

	public function controlsOverVideo(fullScreenOn:Boolean):Boolean {
		if (this.config.controlsOverVideo == 'ease') return true;
		if (this.config.controlsOverVideo == 'no') return false;
		if (lockControls()) return true;
		return getChild().controlsOverVideo(isFullScreen());
	}
	
	public function lockControls():Boolean {
		if (this.config.controlsOverVideo == undefined) return getChild().lockControls();
		return this.config.controlsOverVideo == 'locked';
	}
	
	public function usePlayOverlay():Boolean {
		if (this.config.usePlayOverlay == undefined) return getChild().usePlayOverlay();
		return this.config.usePlayOverlay;
	}
	
	public function getControlBarGloss():String {
		if (this.config.controlBarGloss == undefined) return getChild().getControlBarGloss();
		return this.config.controlBarGloss;
	}

	public function getWatermarkUrl() : String {
		if (this.config.watermarkUrl == undefined) return getChild().getWatermarkUrl();
		return this.config.watermarkUrl;
	}

	public function showWatermark() : String {
 		if (this.config.showWatermark == undefined) return getChild().showWatermark();
		return this.config.showWatermark;
	}
	
	public function getWatermarkLinkUrl():String {
		if (this.config.watermarkLinkUrl == undefined) return getChild().getWatermarkLinkUrl();
		return this.config.watermarkLinkUrl;
	}

}
